@@ -1,11 +1,13 @@
-#include "phylip.h"
-#include "cont.h"
-
 /* version 3.6. (c) Copyright 1993-2004 by the University of Washington.
    Written by Joseph Felsenstein, Akiko Fuseki, Sean Lamont, and Andrew Keeffe.
    Permission is granted to copy and use this program provided no fee is
    charged for it and provided that this copyright notice is not removed. */
 
+#include <float.h>
+
+#include "phylip.h"
+#include "cont.h"
+
 
 #define epsilon1        0.000001   /* small number */
 #define epsilon2        0.02   /* not such a small number */
@@ -157,6 +159,7 @@ void getoptions()
 #ifdef WIN32
     phyFillScreenColor();
 #endif
+    fflush(stdout);
     scanf("%c%*[^\n]", &ch);
     getchar();
     uppercase(&ch);
@@ -290,7 +293,10 @@ void getalleles()
     for (i = 1; i <= loci; i++) {
       if (eoln(infile)) 
         scan_eoln(infile);
-      fscanf(infile, "%ld", &alleles[i - 1]);
+      if (fscanf(infile, "%ld", &alleles[i - 1]) != 1) {
+        printf("ERROR: Unable to read number of alleles at locus %ld\n", i);
+        exxit(-1);
+      }
       if (alleles[i - 1] <= 0) {
     printf("ERROR: Bad number of alleles: %ld at locus %ld\n", alleles[i-1], i);
         exxit(-1);
@@ -328,7 +334,7 @@ void getalleles()
 
 void inputdata()
 { /* read species data */
-  long i, j, k, l, m, n, p;
+  long i, j, k, l, m, m0, n, p;
   double sum;
 
   if (printdata) {
@@ -375,6 +381,7 @@ void inputdata()
     m = 1;
     p = 1;
     for (j = 1; j <= loci; j++) {
+      m0 = m;
       sum = 0.0;
       if (contchars)
         n = 1;
@@ -385,7 +392,11 @@ void inputdata()
       for (k = 1; k <= n; k++) {
         if (eoln(infile)) 
           scan_eoln(infile);
-        fscanf(infile, "%lf", &x[i][m - 1]);
+        if (fscanf(infile, "%lf", &x[i][m - 1]) != 1) {
+          printf("ERROR: unable to read allele frequency"
+              "for species %ld, locus %ld\n", i+1, j);
+          exxit(-1);
+        }
         sum += x[i][m - 1];
         if (!contchars && x[i][m - 1] < 0.0) {
           printf("\n\nERROR: locus %ld in species %ld: an allele", j, i+1);
@@ -409,9 +420,9 @@ void inputdata()
       "\n\nERROR: Locus %ld in species %ld: frequencies do not add up to 1\n",
                   j, i + 1);
           printf("\nFrequencies are:\n");
-          for (l = 0; l <= m-3; l++)
-            printf("%lf+", x[i][l]);
-          printf("%lf = %lf\n\n", x[i][m-2], sum);
+          for (l = m0; l <= m-3; l++)
+            printf("%f+", x[i][l]);
+          printf("%f = %f\n\n", x[i][m-2], sum);
           exxit(-1);
         } else {
           for (l = 0; l <= m-2; l++)
@@ -429,9 +440,9 @@ void inputdata()
             printf("\n\nERROR: Locus %ld in species %ld: ", j, i + 1);
             printf("frequencies add up to more than 1\n");
             printf("\nFrequencies are:\n");
-            for (l = 0; l <= m-3; l++)
-              printf("%lf+", x[i][l]);
-            printf("%lf = %lf\n\n", x[i][m-2], sum);
+            for (l = m0-1; l <= m-3; l++)
+              printf("%f+", x[i][l]);
+            printf("%f = %f\n\n", x[i][m-2], sum);
             exxit(-1);
           }
         }
@@ -475,11 +486,11 @@ void transformgfs()
       if ((pbar[j]*x[i][j]) >= 0.0)
         sumprod[locus[j]-1] += sqrtp[j]*sqrt(x[i][j]);
     for (j = 0; j < totalleles; j++) { /* the projection to tangent plane */
-      f = 1.0 + sumprod[locus[j]-1];
+      f = (1.0 + sumprod[locus[j]-1])/2.0;
       if (x[i][j] == 0.0)
         x[i][j] = (2.0/f - 1.0)*sqrtp[j];
       else
-        x[i][j] = (2.0/f)*sqrt(x[i][j]) + (2.0/f - 1.0)*sqrtp[j];
+        x[i][j] = (1.0/f)*sqrt(x[i][j]) + (1.0/f - 1.0)*sqrtp[j];
     }
   }
   maxalleles = 0;
@@ -493,12 +504,12 @@ void transformgfs()
   for (j = 0; j < loci; j++) {            /* do this for each locus */
     for (k = 0; k < alleles[j]-1; k++) {  /* one fewer than # of alleles */
       c[k][0] = 1.0;
-      for (l = 0; l < k; l++) {           /* for contrasts 1 to k make it ... */
+      for (l = 0; l < k; l++) {          /* for contrasts 1 to k make it ... */
         sum = 0.0;
         for (n = 0; n <= l; n++)
           sum += c[k][n]*c[l][n];
-        if (fabs(c[k-1][l]) > 0.000000001) /* ... orthogonal to those ones */
-          c[k][l+1] = -sum / c[k-1][l];    /* set coeff to make orthogonal */
+        if (fabs(c[l][l+1]) > 0.000000001) /* ... orthogonal to those ones */
+          c[k][l+1] = -sum / c[l][l+1];    /* set coeff to make orthogonal */
         else
           c[k][l+1] = 1.0;
       }
@@ -507,12 +518,16 @@ void transformgfs()
         sum += c[k][l]*sqrtp[m+l];
       if (sqrtp[m+k+1] > 0.0000000001)
         c[k][k+1] = - sum / sqrtp[m+k+1];  /* ... setting last coeff */
-      else
-        c[k][k+1] = 0.0;
+      else {
+        for (l = 0; l <= k; l++)
+          c[k][l] = 0.0;
+        c[k][k+1] = 1.0;
+      }
       sum = 0.0;
       for (l = 0; l <= k+1; l++)
         sum += c[k][l]*c[k][l];
-      for (l = 0; l <= k+1; l++)      /* normalize coeffs so orthonormal */
+      sum = sqrt(sum);
+      for (l = 0; l <= k+1; l++)
         if (sum > 0.0000000001)
           c[k][l] /= sum;
     }
@@ -1325,7 +1340,8 @@ void maketree()
   long i;
 
   if (usertree) {
-    openfile(&intree,INTREE,"input tree file", "r",progname,intreename);
+    /* Open in binary: ftell() is broken for UNIX line-endings under WIN32 */
+    openfile(&intree,INTREE,"input tree file", "rb",progname,intreename);
     numtrees = countsemic(&intree);
     if(numtrees > MAXSHIMOTREES)
       shimotrees = MAXSHIMOTREES;
@@ -1398,7 +1414,7 @@ void maketree()
     while (nextsp <= spp) {
       buildnewtip(enterorder[nextsp - 1], &curtree, nextsp);
       copy_(&curtree, &priortree);
-      bestree.likelihood = -99999.0;
+      bestree.likelihood = -DBL_MAX;
       addtraverse(curtree.nodep[enterorder[nextsp - 1] - 1]->back,
                   curtree.start, true );
       copy_(&bestree, &curtree);
