@@ -112,6 +112,7 @@ void getoptions()
 #ifdef WIN32
     phyFillScreenColor();
 #endif
+    fflush(stdout);
     scanf("%c%*[^\n]", &ch);
     getchar();
     if (ch == '\n')
@@ -139,6 +140,7 @@ void getoptions()
 #ifdef WIN32
               phyFillScreenColor();
 #endif
+              fflush(stdout);
               scanf("%ld%*[^\n]", &numtrees);
               getchar();
               done1 = (numtrees >= 1);
@@ -201,9 +203,14 @@ void getdata()
     scan_eoln(infile);
     initname(i);
     if (varywithin) {
-      fscanf(infile, "%ld", &sample[i]);
-      contnum += sample[i]-1;
-      scan_eoln(infile);
+      if (fscanf(infile, "%ld", &sample[i]) == 1) {
+        contnum += sample[i]-1;
+        scan_eoln(infile);
+      }
+      else {
+        printf("Error reading number of individuals for species %ld\n", i+1);
+        exxit(-1);
+      }
     }
     else sample[i] = 1;
     if (printdata)
@@ -218,7 +225,10 @@ void getdata()
       for (j = 1; j <= chars; j++) {
         if (eoln(infile)) 
           scan_eoln(infile);
-        fscanf(infile, "%lf", &x[i][k][j - 1]);
+        if (fscanf(infile, "%lf", &x[i][k][j - 1]) != 1) {
+          printf("Error in input file at species %ld\n", i+1);
+          exxit(-1);
+        }
         if (printdata) {
           fprintf(outfile, " %9.5f", x[i][k][j - 1]);
           if (j % 6 == 0) {
@@ -904,7 +914,8 @@ int main(int argc, Char *argv[])
 #endif
   init(argc, argv);
   openfile(&infile,INFILE,"input data","r",argv[0],infilename);
-  openfile(&intree,INTREE,"input tree", "r",argv[0],intreename);
+  /* Open in binary: ftell() is broken for UNIX line-endings under WIN32 */
+  openfile(&intree,INTREE,"input tree", "rb",argv[0],intreename);
   openfile(&outfile,OUTFILE,"output", "w",argv[0],outfilename);
   ibmpc = IBMCRT;
   ansi = ANSICRT;
