@@ -135,7 +135,7 @@ void getoptions()
       case MAT_MACHINE:
         str = "Square";
         break;
-      case MAT_HUMAN | MAT_LOWER:
+      case MAT_LOWERTRI:
         str = "Lower-triangular";
         break;
       case MAT_HUMAN:
@@ -237,18 +237,16 @@ void getoptions()
        /* square -> lower-triangular -> machine-readable */
        switch ( matrix_flags ) {
          case MAT_HUMAN:
-           matrix_flags |= MAT_LOWER;
+           matrix_flags = MAT_MACHINE;
            break;
-         case MAT_HUMAN | MAT_LOWER:
-           if ( similarity )
-             matrix_flags = MAT_HUMAN;
-           else 
-             matrix_flags = MAT_MACHINE;
-           break;
-         case MAT_MACHINE:
+         case MAT_LOWERTRI:
            matrix_flags = MAT_HUMAN;
            break;
+         case MAT_MACHINE:
+           matrix_flags = MAT_LOWERTRI;
+           break;
          default:
+           assert(0);
            matrix_flags = MAT_HUMAN;
        }
        break;
@@ -312,8 +310,16 @@ void getoptions()
 
   /* Prevent similarity matrices from easily being used as input to other
    * programs */
-  if (similarity && matrix_flags == MAT_MACHINE)
-    matrix_flags = MAT_HUMAN;
+  if (similarity) {
+    if (matrix_flags == MAT_MACHINE)
+      matrix_flags = MAT_HUMAN;
+    else if (matrix_flags == MAT_LOWERTRI)
+      matrix_flags = MAT_LOWER | MAT_HUMAN;
+  }
+
+#ifdef DEBUG
+  fprintf(stderr, "Matrix type: %x %s\n", matrix_flags);
+#endif /* DEBUG */
 
   if (gama || invar) {
     loopcount = 0;
