@@ -78,6 +78,13 @@ extern long  HiMode,GraphDriver,GraphMode,LoMode,bytewrite;
 void handlemouse(WindowPtr win,EventRecord ev);
 
 /* Global variables used in many functions*/
+
+#ifdef OSX_CARBON
+/* Process Serial Number in text mode. Typically the OSX Terminal application */
+ProcessSerialNumber textModePSN;
+Boolean textModePSN_valid = false;
+#endif 
+
 int mode = TEXT;
 int quitmac;
 WindowPtr gfx_window;
@@ -99,7 +106,7 @@ void
 macsetup (char *tname, char *gname)
 {
   Str255 buf1, buf2;                        
-  Str255 title = "";        
+  /* Str255 title = ""; */
   Rect plot_rec={10,10,30,110}; 
   Rect change_rec={10,120,60,220};
   Rect quit_rec={40,10,60,110};
@@ -108,17 +115,27 @@ macsetup (char *tname, char *gname)
   unsigned char* change=(unsigned char*)"\7Change\rParameters";
   unsigned char* quit=(unsigned char*)"\4Quit";
   unsigned char* about=(unsigned char*)"\5About";
+
 #ifndef __MWERKS__
   ProcessSerialNumber myProcess;
 #endif
+
 #ifdef OSX_CARBON
+  OSStatus err;
   BitMap screenBits;
+
+  /* Save the frontmost process, which should be the process containing the console. */
+  err = GetFrontProcess(&textModePSN);
+  if (err != noErr)
+    printf("Cannot get PSN of front process: %d\n", err);
+
   GetQDGlobalsScreenBits(&screenBits);
 #endif
 
 #ifndef OSX_CARBON
   change[0]=0x11;
 #endif
+
   background.red=0xcc00; /* #ccffff phylip color */
   background.green=0xffff;
   background.blue=0xffff;
@@ -128,13 +145,13 @@ macsetup (char *tname, char *gname)
 #ifndef OSX_CARBON  /* this looks bad in OS X */
   SIOUXSettings.fontsize= log(qd.screenBits.bounds.right);
 #endif
+
 #ifdef __MWERKS__
   SIOUXSettings.autocloseonquit = true;
 #endif
 
   putchar('\n'); /* initialize sioux and let sioux initialize the toolbox and menus*/
   
-  
   strcpy ((char *) buf1 + 1, tname);
   strcpy ((char *) buf2 + 1, gname);
   buf1[0] = strlen (tname);
@@ -165,6 +182,7 @@ macsetup (char *tname, char *gname)
   gfx_window = NewCWindow (0L, &gfxBounds, buf2, false, documentProc,
                            (WindowPtr) - 1L, true, 0);
 #endif
+
   plot_button = NewControl(gfx_window,&plot_rec,plot,1,0,0,1,pushButProc,0);
   change_button = NewControl(gfx_window,&change_rec,change,1,0,0,1,pushButProc,0);
   quit_button = NewControl(gfx_window,&quit_rec,quit,1,0,0,1,pushButProc,0);
@@ -294,12 +312,47 @@ putstring (string)
 void
 textmode ()
 {
+#ifdef OSX_CARBON
+  ProcessSerialNumber psn;
+  const Boolean visible = false;
+  OSErr err;
+  ProcessInfoRec info;
+  unsigned char processName[32];
+
+  memset(processName, '\0', 32);
+
+#  ifdef DEBUG
+  /* Print the process name which we have saved. (Should be "Terminal" or somesuch.) */
+  if (textModePSN_valid == false)
+    assert(0);
+
+  info.processInfoLength = sizeof(ProcessInfoRec);
+  info.processName = processName;
+  info.processAppSpec = NULL;
+
+  err = GetProcessInformation(&textModePSN, &info);
+  if (err == noErr) {
+    printf("Text mode process: \"%s\" %d\n", info.processName + 1, info.processName[0]);
+  }
+#  endif
+
+  /* Bring the terminal process to the front again */
+  
+  err = SetFrontProcess(&textModePSN);
+  if (err != noErr)
+    printf("Warning: cannot bring text-mode process to front: %d\n", err);
+#endif
+
+  /* Hide the graphics window */
   HideWindow (gfx_window);
+
   mode = TEXT;
 }
 
 /* go into graphics mode */
 void gfxmode() {
+  OSErr err;
+  ProcessSerialNumber psn;
 
   InitCursor();
 
@@ -309,9 +362,17 @@ void gfxmode() {
   SetPort(gfx_window);
 #endif
 
+  /* Display the graphics window and bring it to the front of the application */
   ShowWindow(gfx_window);
   SelectWindow(gfx_window);
 
+  /* Bring our process to the foreground, in case we were operating from Terminal.app */
+  err = GetCurrentProcess(&psn);
+  if (err == noErr) {
+    ShowHideProcess(&psn, true);
+    SetFrontProcess(&psn);
+  }
+
   mode = GFX;
 }
 
@@ -387,7 +448,7 @@ void do_about () {
   short txt;
   
   Rect gfxBounds = {50,50,300,600};
-  Rect labelRect = {25,15,150,400};
+  /* Rect labelRect = {25,15,150,400}; */
   Rect buttonRect = {200,350,220,450};
 #ifdef OSX_CARBON
   CreateNewWindow (kMovableModalWindowClass,0, &gfxBounds, &about_dialog);
@@ -416,7 +477,7 @@ void do_about () {
 
 
 void paint_dialog(WindowPtr about_dialog) {
-  int i;
+  /* int i; */
   char *cur;
   char *what,*tmpstr,*orig;
   int go=1;
@@ -463,7 +524,7 @@ void handle_dlg_event(WindowPtr dlg)
   ControlHandle h;
   int go=1;
   short w;
-  int i;
+  /* int i; */
   Point mouse;
 
   while (go) {
