@@ -169,6 +169,7 @@ void getoptions()
 #ifdef WIN32
     phyFillScreenColor();
 #endif
+    fflush(stdout);
     scanf("%c%*[^\n]", &ch);
     getchar();
     if (ch == '\n')
@@ -229,6 +230,7 @@ void getoptions()
 #ifdef WIN32
             phyFillScreenColor();
 #endif
+            fflush(stdout);
             scanf("%c%*[^\n]", &ch2);
             getchar();
             if (ch2 == '\n')
@@ -1020,7 +1022,8 @@ void maketree()
       }
     }
   } else {
-    openfile(&intree,INTREE,"input tree file", "r",progname,intreename);
+    /* Open in binary: ftell() is broken for UNIX line-endings under WIN32 */
+    openfile(&intree,INTREE,"input tree file", "rb",progname,intreename);
     numtrees = countsemic(&intree);
     if (numtrees > 2)
       initseed(&inseed, &inseed0, seed);
