@@ -1871,7 +1871,7 @@ void makedists()
       for (j = 1; j <= k; j++) {
         if (d[i][j-1] < 100.0)
           fprintf(outfile, "%10.6f", d[i][j-1]);
-        else if (d[j][k] < 1000.0)
+        else if (d[i][j-1] < 1000.0)
           fprintf(outfile, " %10.6f", d[i][j-1]);
           else 
             fprintf(outfile, " %11.6f", d[i][j-1]);
@@ -1970,6 +1970,12 @@ int main(int argc, Char *argv[])
 #ifdef MAC
   fixmacfile(outfilename);
 #endif
+  printf("Done.\n\n");
+
+#ifdef WIN32
+  phyRestoreConsoleAttributes();
+#endif
+
   return 0;
 }  /* Protein distances */
 
