@@ -1925,6 +1925,11 @@ int main(int argc, Char *argv[])
 #ifdef MAC
   fixmacfile(outfilename);
   fixmacfile(outtreename);
+#endif
+  printf("Done.\n\n");
+
+#ifdef WIN32
+  phyRestoreConsoleAttributes();
 #endif
   return 0;
 }  /* Protein parsimony by uphill search */
