@@ -2463,5 +2463,8 @@ int main(int argc, Char *argv[])
   fixmacfile(outtreename);
 #endif
   printf("Done.\n\n");
+#ifdef WIN32
+  phyRestoreConsoleAttributes();
+#endif
   return 0;
 }  /* maximum likelihood phylogenies from restriction sites */
